@@ -18,15 +18,18 @@ BASIC_MAPPINGS = {
     dt.datetime: "datetime",
 }
 
+
 class _Types:
     NONETYPE = type(None)
 
+
 class TypeDAL(pydal.DAL):
     """@DynamicAttrs"""
+    dal: Table
 
     def define(self, cls):
         tablename = self._to_snake(cls.__name__)
-        return self.define_table(
+        table = self.define_table(
             tablename,
             *[self._to_field(fname, ftype)
 
@@ -34,6 +37,14 @@ class TypeDAL(pydal.DAL):
               ]
         )
 
+        # todo: document:
+        table.cls = self
+        self.dal = table
+
+        return table
+
+    # todo: insert etc shadowen?
+
     @classmethod
     def _to_field(cls, fname, ftype):
         fname = cls._to_snake(fname)
@@ -83,19 +94,30 @@ class TypedRows(Rows):
 
 
 class TypedFieldType(Field):
+    _table = "<any table>"
+
     def __init__(self, _type, **kwargs):
         self.type = _type
         self.kwargs = kwargs
 
     def __repr__(self):
-        t = self.type.__name__ if type(self.type) is type else self.type
-        return f"<TypedField {t} with options {self.kwargs}>"
+        s = self.__str__()
+        return f"<{s} with options {self.kwargs}>"
+
+    def __str__(self):
+        if 'type' in self.kwargs:
+            t = self.kwargs['type']
+        else:
+            t = self.type.__name__ if type(self.type) is type else self.type
+        return f"TypedField.{t}"
 
     def to_field(self, name: str) -> Field:
-        _type = self._to_field_type(self.type)
-        if 'type' in self.kwargs:
-            _type = self.kwargs.pop('type')
-        return Field(name, _type, **self.kwargs)
+        other_kwargs = self.kwargs.copy()
+        if 'type' in other_kwargs:
+            _type = other_kwargs.pop('type')
+        else:
+            _type = self._to_field_type(self.type)
+        return Field(name, _type, **other_kwargs)
 
     @classmethod
     def _to_field_type(cls, _type: type) -> str:
